#include <sys/types.h>
#include <dirent.h>
#include <syslog.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <stdlib.h>
#include <pwd.h>

int mhd();
void start_seti();
void kill_seti();

/*
 * /usr/monash/ses/cronjob
 *
 * expected to run once a minute
 * it scans /tmp looking for TGT files.
 * for each one it finds, it checks to see if the owner has any processes
 * if so it sends a MountHomeDirectory refresh
 * else  for security it deletes the ticket file, and unmounts the directory
 * finally it starts or stops SETI depending on whether or not it found
 * anyone using the machine.
 */

int main(){

  int found, anyfound;
  DIR *td, *pd;
  struct dirent *d, *pp;
  uid_t uid;
  struct stat s;
  time_t now;
  char *fn, *pn;
  struct passwd *pwd;

  openlog( "SEScronJob", LOG_CONS,LOG_AUTH);

  anyfound = 0;

  /* open /tmp */
  if ( ! (td = opendir("/tmp")) ){
    syslog( LOG_ERR, "SES can't open /tmp");
    exit(-1);
  }
 
  /* 
   *read through /tmp looking for krb5cc_uid_whatever
   */

  while ( (d = readdir(td)) ){
    if ( ! strstr(d->d_name, "krb5cc_") ) continue;

    found = 0;

    fn = (char *) malloc(strlen(d->d_name)+6);
    if ( fn == (char *) 0 ) {
      syslog( LOG_ERR, "SES can't malloc fn %m");
      exit(-1);
    }
    strcpy(fn,"/tmp/");
    strcat(fn,d->d_name);

    /*
     * if its less than 3 minutes old, leave it to PAM to handle
     */

    if ( stat( fn, &s ) ) {
      syslog( LOG_ERR, "SES can't stat %s",fn);
      exit(-1);
    }

    if ( ( time(0) - s.st_ctime ) < (3 * 60 ) ) {
      anyfound = 1;
      continue;
    }

    /*
     * if the owner of this file has any processes running
     * set found and send a refresh
     */

    if ( ! sscanf(fn, "/tmp/krb5cc_%d_",&uid) ) {
      syslog( LOG_ERR, "SES can't scan uid from %s",fn);
      exit(-1);
    }
    
    /* ignore root tickets - they shouldn't be here */
    if ( uid == 0 ) continue;

    /* open /proc */
    if ( ! (pd = opendir("/proc")) ){
      syslog( LOG_ERR, "SES can't open /proc");
      exit(-1);
    }
    while ( (pp = readdir(pd)) ) {

      pn = (char *) malloc(strlen(pp->d_name)+7);
      if ( pn == (char *) 0 ) {
        syslog( LOG_ERR, "SES can't malloc pn %m");
        exit(-1);
      }
      strcpy(pn,"/proc/");
      strcat(pn,pp->d_name);

      if ( stat( pn, &s ) ) {
        syslog( LOG_ERR, "SES can't stat %s",pn);
        exit(-1);
      }

      free(pn);

      if ( s.st_uid == uid ) {
	found = 1;
        if ( setenv("KRB5CCNAME", fn, 1) ){
          syslog( LOG_ERR,"SES setenv error");
	  exit(-1);
	} 
	if ( mhd() ){
	  syslog( LOG_ERR,"SES MakeHomeDir error");
	  /* don't exit here, it was prolly just expired tickets */
          /* but this may be a bit ambitious */
          found = 0;
	}
	break;
      }
    }

    if ( found ) {
      anyfound = 1;
    }
    else {

      /*
       * we found a krb5cc_uid_stuff file, 
       * it was older than 3 minutes
       * and no processes were found for that uid
       * so we remove the krb5cc_uid_thing file.
       * that way no hackers can use it
       * also unmount the directory for the same reason.
       */

      if ( unlink( fn ) ) {
        syslog( LOG_ERR,"SES error unlinking %s", fn);
      }

      if ( (pwd = getpwuid(uid)) == (struct passwd *) 0 ) {
        syslog( LOG_ERR,"SES error getting dir from uid %m");
      }

      if ( umount(pwd->pw_dir) ){
	syslog( LOG_ERR,"SES error unmounting %s %m",pwd->pw_dir);
      }

    }

    free(fn);

  }
  
  /*
   * We've scanned all of /tmp, either refreshing the mhd or
   * removing the ticket file.
   * Now, if we found (and refreshed) a user kill seti if
   * it is indeed running, 
   * otherwise if we didn't find a user, start seti.
   */
  exit(0);
  if ( anyfound) kill_seti();
  else start_seti();
  exit(0);
}
