
/* 
 *
 * kill_seti.c
 * 
 *   if seti is running, kill it, and unmount the disk
 *
 */


#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>
#include <sys/mount.h>

void kill_seti() {

  pid_t pid;
  int pf;
  char buf[128];
  int len;

  pf = open("/var/run/seti.pid",O_RDONLY);
  if ( ( pf < 0 ) && ( errno != ENOENT ) ) {
    syslog( LOG_ERR, "kill_seti can't open pidfile %m");
    return;
  }
  if ( pf < 0 ) {
    /* if a seti is running, it isn't ours. do an unmount anyway */
    system( "umount /seti" );
    return;  
  }
  len = read( pf, buf, 128);
  close(pf);
  if ( len <= 0 ) {
    syslog( LOG_ERR, "kill_seti can't read pidfile %m");
    return;
  }
  buf[len] = '\0';
  if ( ! sscanf( buf, "%d", &pid ) ) {
    syslog( LOG_ERR, "kill_seti can't find pid in pidfile");
    return;
  }
  if ( kill( pid, SIGTERM) ) {
    syslog( LOG_ERR, "kill_seti can't kill %d %m",pid);
    return;
  }
  if ( unlink("/var/run/seti.pid") ) {
    syslog( LOG_ERR,"kill_seti can't unlink pid file %m");
    return;
  }
  sleep(2);
  if ( system( "umount /seti") ) {
    syslog( LOG_ERR, "kill_seti can't umount /seti %m");
    return;
  }
}
