/* Modified from pam_al.c by Gerald Britton <gbritton@mit.edu> */
/* by Katherine Lim <katherine.lim@its.monash.edu.au>          */
/* 30 June 2000 Initial version                                */
/* Calls mhd by Keith Lewis <keith.lewis@its.monash.edu.au>    */
/* int mhd (char *user)                                        */

#define PAM_SM_SESSION

#include <syslog.h>
#include <security/pam_modules.h>
#include <stdio.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/mount.h>

/* Session open/close. */
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
	const char **argv); 
PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
	const char **argv); 

/* Logging macros. */
#define MODULENAME "pam_mhd"
#define DEBUG(x) if(debug) syslog(LOG_DEBUG, MODULENAME ": " x)
#define DEBUG1(x,y) if(debug) syslog(LOG_DEBUG, MODULENAME ": " x,y)
#define DEBUG2(x,y,z) if(debug) syslog(LOG_DEBUG, MODULENAME ": " x,y,z)
#define WARN(x) syslog(LOG_NOTICE, MODULENAME ": " x)
#define WARN1(x,y) syslog(LOG_NOTICE, MODULENAME ": " x,y)
#define WARN2(x,y,z) syslog(LOG_NOTICE, MODULENAME ": " x,y,z)
#define WARN3(w,x,y,z) syslog(LOG_NOTICE, MODULENAME ": " w,x,y,z)

PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
	const char **argv)
{
	char *user;
	int mhd_ret;
	int sleeptime;

    if ((pam_get_item(pamh, PAM_USER, (void *)&user)) != PAM_SUCCESS) {
        WARN("could not get pam username");
        return PAM_SESSION_ERR;
    }

    if ( (strcmp (user, "root") ) == 0 ) {
        WARN("not attempting to mount home directory for root");
        return PAM_SUCCESS;
    }

    WARN1("attempting to mount home directory for %s", user);
    sleeptime = 0;
    while ( ( mhd_ret = mhd(user) ) != 0 ) sleep(sleeptime++);
    WARN1("status: mhd returned %d", mhd_ret);

	return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
	const char **argv)
{
    char *user;
    struct passwd *my_passwd;
    char buf[1024];
    int umount_ret;
    umount_ret = 0;

    if ((pam_get_item(pamh, PAM_USER, (void *)&user)) != PAM_SUCCESS) {
        WARN("could not get pam username");
        return PAM_SESSION_ERR;
    }
    
    if ( (strcmp (user, "root") ) == 0 ) {
        WARN("not attempting to unmount home directory for root");
        return PAM_SUCCESS;
    }    

    if ( (my_passwd = getpwnam(user)) == (struct passwd *) 0 ) {
        WARN1("error looking up %s", user);
        return PAM_SESSION_ERR;
    }

/* need code to update mtab as well so we will just sys umount for now */
/*    umount_ret = umount (my_passwd->pw_dir); */
    sprintf(buf,"umount %s",my_passwd->pw_dir);
    umount_ret = system(buf);
    switch (umount_ret) {
    case 0:
        WARN1("successfully unmounted %s", my_passwd->pw_dir);
        break;
    default:
        WARN2("error %d while unmounting %s", umount_ret, my_passwd->pw_dir);
    }

    

    return PAM_SUCCESS;
}
